Attribute VB_Name = "PrinterFunctions"
Option Explicit

            'one of six windows structures designed to control printers
            '#5 is used under windows 95, althought, you can use other structures
Type PRINTER_INFO_5
        pPrinterName As String
        pPortName As String
        Attributes As Long
        DeviceNotSelectedTimeout As Long
        TransmissionRetryTimeout As Long
End Type

            'This function gets information about a specific printer
            'Printer must be opened with the OpenPrinter function
Declare Function GetPrinter Lib "winspool.drv" Alias "GetPrinterA" _
(ByVal hPrinter As Long, ByVal Level As Long, pPrinter As Any, _
ByVal cbBuf As Long, pcbNeeded As Long) As Long

            'This function allows you to change information regarding a specific printer
Declare Function SetPrinter Lib "winspool.drv" Alias "SetPrinterA" _
(ByVal hPrinter As Long, ByVal Level As Long, pPrinter As Any, _
ByVal Command As Long) As Long

            'This function copies string (needed to move data from buffer to the structure
Declare Function LstrCpy Lib "kernel32" Alias "lstrcpyA" _
(ByVal lpString1 As String, ByVal lpString2 As Any) As Long

            'This function displays an error number for the most recent error
Declare Function GetLastError& Lib "kernel32" ()

            'This function closes the current handle to a printer
Public Declare Function ClosePrinter& Lib "winspool.drv" (ByVal hPrinter As Long)

            'This function obtains a handle to a specified printer.
            'This handle is required in order to call all other function
Public Declare Function OpenPrinter& Lib "winspool.drv" Alias "OpenPrinterA" (ByVal pPrinterName As String, phPrinter As Long, ByVal pDefault As Long)

            'This attribute is being set in the structure to put the printer back online
Public Const PRINTER_ATTRIBUTE_WORK_OFFLINE = &H400

Public Sub EnablePrinter(prnName As String, Optional SetOffline As Boolean = False)
  'This function takes two parameters. One is the name of the
  'Printer name, just like it appears in the printer manager
  'Second is an optional parameter. If set to true, will take printer offline.
  
  Dim BufferSize As Long
  Dim pInfo5 As PRINTER_INFO_5
  Dim res As Long
  Dim Buffer() As Long
  Dim hPrinter As Long

    On Error GoTo EnablePrinter_Error

    If prnName = "" Then Exit Sub           'Make sure that we have a printer to work with

    res = OpenPrinter(prnName, hPrinter, 0) 'Obtain a handle to a printer
    If res = 0 Then GoTo DidNotSwitch       'If res <> 0 then Error occured, 0 if success

                            ' Find out how large the buffer needs to be
    res = GetPrinter(hPrinter, 5, 0, 0, BufferSize)    'find out how big the buffer should be
    
    ReDim Buffer(BufferSize) As Long                    'resize the buffer
    
    res = GetPrinter(hPrinter, 5, Buffer(0), BufferSize, BufferSize)    'fills the buffer
    If res = 0 Then GoTo DidNotSwitch                                   'with data. Will later go to Printer_Info_5
           
    pInfo5.pPrinterName = ConvertToVbString(Buffer(0))     'Copy from buffer into the structure
    pInfo5.pPortName = ConvertToVbString(Buffer(1))        'use a special function for that task
    pInfo5.Attributes = Buffer(2)
    pInfo5.DeviceNotSelectedTimeout = Buffer(3)
    pInfo5.TransmissionRetryTimeout = Buffer(4)
        
    If SetOffline = False Then
        pInfo5.Attributes = 0                               'This will reset the printer attributes, and put the printer back online
    Else
        pInfo5.Attributes = PRINTER_ATTRIBUTE_WORK_OFFLINE
    End If
    
    res = SetPrinter(hPrinter, 5, pInfo5, 0)            'Change the printer settings
    If res = 0 Then GoTo DidNotSwitch

    res = ClosePrinter(hPrinter)                        'Done with this printer
    If res = 0 Then GoTo DidNotSwitch

    Exit Sub                                            'Success
    
DidNotSwitch:
    MsgBox "Did not switch printer successfully"        'Failed

    Exit Sub
    
EnablePrinter_Error:                                    'Failed
    MsgBox Error

End Sub


Private Function ConvertToVbString(lngStr As Long) As String

 On Error Resume Next

 Dim strTemp As String * 512
 Dim i As Long

 i = LstrCpy(strTemp, lngStr)                   'Convert String
 
 If (InStr(1, strTemp, Chr(0)) = 0) Then          'Check for zeros
    ConvertToVbString = ""
 Else
    ConvertToVbString = Left(strTemp, InStr(1, strTemp, Chr(0)) - 1)    'Return VB string
 End If
 
End Function

